/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* 
 * Test application for waaad.
 */

#define DECLARE_API_POINTERS
#include "app_test.h"

/* Initialize the configuration */
atst_conf_t * atst_conf = NULL;
static atst_conf_t _conf;

static int conf_init(void)
{
	atst_conf = &_conf;
	memset(atst_conf, 0, sizeof(atst_conf_t));
	
	/* Set the default values */
	atst_conf->vendor_id  = 999999;
	atst_conf->appli_id   = 123456;
	atst_conf->cmd_id     = 234567;
	atst_conf->avp_id     = 345678;
	atst_conf->mode       = MODE_SERV | MODE_CLI;
	atst_conf->dest_realm = strdup(g_pconf->diameter_realm);
	atst_conf->dest_host  = NULL;
	atst_conf->signal     = 10; /* SIGUSR1 */
	
	return 0;
}

/* entry point */
static int entry(char * conffile)
{
	TRACE_ENTRY("%p", conffile);
	
	/* Initialize configuration */
	CHECK( conf_init() );
	
	/* Parse configuration file */
	if (conffile != NULL) {
		CHECK( conf_parse(conffile) );
	}
	
	/* Install objects definitions for this test application */
	CHECK( dict_init() );
	
	/* Install the handlers for incoming messages */
	if (atst_conf->mode & MODE_SERV) {
		CHECK( serv_init() );
	}
	
	/* Start the signal handler thread */
	if (atst_conf->mode & MODE_CLI) {
		CHECK( cli_init() );
	}
	
	return 0;
}

/* Unload */
void waaad_ext_fini(void)
{
	if (atst_conf->mode & MODE_CLI)
		cli_fini();
	if (atst_conf->mode & MODE_SERV)
		serv_fini();
}

EXTENSION_API_INIT( API_MODULE_ALL, entry, "app_test");
